<?php

namespace EventPortal\Domain\Component;

use EventPortal\Domain\Field\Enums\FieldType;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;
use ReflectionProperty;

abstract class Component
{
    /** @var string */
    public $type;

    /** @var null|string */
    public $name;

    /** @var string */
    public $label;

    /** @var null|string */
    public $classes;

    public function __construct(?string $label = null)
    {
        $this->label = $label;
        $this->name = $label !== null ? Str::slug($label, '_') : null;
        $this->type = $this->type();
    }

    public static function make(?string $label = null): self
    {
        return new static($label);
    }

    public function name(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function label(string $label): self
    {
        $this->label = $label;

        return $this;
    }

    /**
     * @param string|array|null $classes
     */
    public function classes($classes): self
    {
        $this->classes = Arr::wrap($classes);

        return $this;
    }

    public function type(): FieldType
    {
        $class = Str::afterLast(get_class($this), '\\');

        return FieldType::$class();
    }

    /**
     * @param mixed $value
     * @return mixed|$this
     */
    public function when($value, callable $callback, ?callable $default = null)
    {
        if ($value) {
            return $callback($this, $value) ?: $this;
        }

        if ($default) {
            return $default($this, $value) ?: $this;
        }

        return $this;
    }

    /**
     * Returns the component as an array.
     */
    public function toArray(): array
    {
        $data = [
            'component' => $this->type,
            'name' => $this->name,
            'label' => $this->label,
        ];

        if ($this->classes !== []) {
            $data['classes'] = $this->classes;
        }

        return $this->withPublicProperties($data);
    }

    private function withPublicProperties(array $data): array
    {
        $properties = (new \ReflectionClass($this))->getProperties(ReflectionProperty::IS_PUBLIC);

        $properties = array_filter($properties, function ($property) use ($data) {
            return $property->class !== self::class;
        });

        foreach ($properties as $property) {
            $value = $property->getValue($this);

            $data[$property->getName()] = $value instanceof Collection ? $value->toArray() : $value;
        }

        return $data;
    }
}
